import csv
import numpy as np
import stan
import arviz as az
import matplotlib.pyplot as plt
import seaborn as sb
import scipy.stats as ss
import japanize_matplotlib

fin_nm = input('Input file name (*.csv) = ')
with open(fin_nm, 'r') as f:
    csv_data = [d for d in csv.reader(f)]

savefig_nm = 'Figure.png'
fout_nm = 'Results.txt' 
fout = open(fout_nm, 'w')
fout.write('Input data file = {}'.format(fin_nm))

N = len(csv_data) - 1
print('N =', N)
X = []
Y = []
Z = []
for i in range(N):
    X.append(float(csv_data[i+1][1]))
    Y.append(float(csv_data[i+1][2]))
    Z.append(float(csv_data[i+1][3]))

fout.write('\n\nData =\n')
for x, y, z in zip(X, Y, Z):
    fout.write(f'{x}  {y}  {z}\n')

#
#         相関係数の計算
#
Data = {'N': len(X), 'X':X, 'Y':Y, 'Z':Z}

def f_init():
    return {'vmuX':0.5, 'vmuY':0.5, 'vmuZ':0.5,
            'vsgmX':0.5, 'vsgmY':0.5, 'vsgmZ':0.5,
            'cl':[[1.0, 0.0, 0.0], [0.0, 1.0, 0.0], [0.0, 0.0, 1.0]] }

with open('LKJCorr.stan') as f_stan:
    sm0 = stan.build(f_stan.read(), data=Data)
fit0 = sm0.sample(num_chains=4, init=[f_init()]*4) 

print('\nMCMC sampling for the correlations started.')

i_data0 = az.from_pystan(posterior=fit0, posterior_model=sm0) 
print(az.summary(i_data0))

fout.write('\nMCMC sampling summary (Three variate normal distribution)\n')
fout.write('{}\n'.format(az.summary(i_data0)))
#
#       偏相関係数の計算
#
print('\nCompiling the stan script for the partial correlation coefficient.')

with open('PartialCorr.stan') as f_stan:
    sm1 = stan.build(f_stan.read(), data=Data)
fit1 = sm1.sample()  

i_data1 = az.from_pystan(posterior=fit1, posterior_model=sm1)
print(az.summary(i_data1))
fout.write('\nMCMC sampling summary (Partial correlation coefficient)\n')
fout.write('{}\n'.format(az.summary(i_data1)))
#
#       （偏）相関係数の出力とグラフ描画
#
RhoXY = fit0['RhoXY']
RhoXZ = fit0['RhoXZ']
RhoYZ = fit0['RhoYZ']
p_corr = fit1['rho']

plt.subplots(figsize = (9, 6))

xcoord = np.linspace(-1, 1, 1000)
ycoord = ss.gaussian_kde(RhoXY).pdf(xcoord)
r_xy_mean = RhoXY.mean()
r_xy_025, r_xy_med, r_xy_975 = np.percentile(RhoXY, [2.5, 50, 97.5])
print('\n相関係数 Rxy')
print('   平均値 = {0:.3f}'.format(r_xy_mean))
print('   中央値 = {0:.3f}'.format(r_xy_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_xy_025, r_xy_975))

fout.write('\nCorrelation Coefficient Rxy\n')
fout.write('  Mean     = {0:.3f}\n'.format(r_xy_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_xy_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_xy_025, r_xy_975))

plt.subplot(2,2,1)
plt.title(('相関係数 ' + r'$\rho_{xy}$') + (': med.={0:.3f}'.format(r_xy_med)),
          fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(p_corr).pdf(xcoord)
p_corr_mean = p_corr.mean()
p_corr_025, p_corr_med, p_corr_975 = np.percentile(p_corr, [2.5, 50, 97.5])
print('\n偏相関係数 Rxy.z')
print('   平均値 = {0:.3f}'.format(p_corr_mean))
print('   中央値 = {0:.3f}'.format(p_corr_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(p_corr_025, p_corr_975))

fout.write('\nPartial Correlation Coefficient Rxy.z\n')
fout.write('  Mean     = {0:.3f}\n'.format(p_corr_mean))
fout.write('  Median   = {0:.3f}\n'.format(p_corr_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(p_corr_025, p_corr_975))

plt.subplot(2,2,2)
plt.title('偏相関係数 ' + r'$\rho_{XY{\bullet}Z}$' +
          ': med.={0:.3f}'.format(p_corr_med), fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(RhoXZ).pdf(xcoord)
r_xz_mean = RhoXZ.mean()
r_xz_025, r_xz_med, r_xz_975 = np.percentile(RhoXZ, [2.5, 50, 97.5])
print('\n相関係数 Rxz')
print('   平均値 = {0:.3f}'.format(r_xz_mean))
print('   中央値 = {0:.3f}'.format(r_xz_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_xz_025, r_xz_975))

fout.write('\nCorrelation Coefficient Rxz\n')
fout.write('  Mean     = {0:.3f}\n'.format(r_xz_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_xz_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_xz_025, r_xz_975))

plt.subplot(2,2,3)
plt.title('相関係数 ' + r'$\rho_{xz}$' + ': med.={0:.3f}'.format(r_xz_med),
          fontsize = 16)
plt.plot(xcoord, ycoord)

ycoord = ss.gaussian_kde(RhoYZ).pdf(xcoord)
r_yz_mean = RhoYZ.mean()
r_yz_025, r_yz_med, r_yz_975 = np.percentile(RhoYZ, [2.5, 50, 97.5])
print('\n相関係数 Ryz')
print('   平均値 = {0:.3f}'.format(r_yz_mean))
print('   中央値 = {0:.3f}'.format(r_yz_med))
print('   95% CI = [{0:.3f}, {1:.3f}]'.format(r_yz_025, r_yz_975))

fout.write('\nCorrelation Coefficient Ryz\n')
fout.write('  Mean     = {0:.3f}\n'.format(r_yz_mean))
fout.write('  Median   = {0:.3f}\n'.format(r_yz_med))
fout.write('  95% CI   = [{0:.3f}, {1:.3f}]\n'.format(r_yz_025, r_yz_975))

plt.subplot(2,2,4)
plt.title('相関係数 ' + r'$\rho_{yz}$' + ': med.={0:.3f}'.format(r_yz_med),
          fontsize = 16)
plt.plot(xcoord, ycoord)

plt.tight_layout()

plt.savefig(savefig_nm)
print('\nFile {} of the figure was saved.'.format(savefig_nm))
plt.show()

fout.close()
print('\nFile {} was saved.'.format(fout_nm))
print()
